@@ -79,8 +79,8 @@ function get_file_description( $file ) {
  * @return string Full filesystem path to the root of the WordPress installation
  */
 function get_home_path() {
-	$home = get_option( 'home' );
-	$siteurl = get_option( 'siteurl' );
+	$home    = set_url_scheme( get_option( 'home' ), 'http' );
+	$siteurl = set_url_scheme( get_option( 'siteurl' ), 'http' );
 	if ( ! empty( $home ) && 0 !== strcasecmp( $home, $siteurl ) ) {
 		$wp_path_rel_to_home = str_ireplace( $home, '', $siteurl ); /* $siteurl - $home */
 		$pos = strripos( str_replace( '\\', '/', $_SERVER['SCRIPT_FILENAME'] ), trailingslashit( $wp_path_rel_to_home ) );
@@ -179,8 +179,8 @@ function validate_file_to_edit( $file, $allowed_files = '' ) {
 		case 1 :
 			wp_die( __( 'Sorry, that file cannot be edited.' ) );
 
-		//case 2 :
-		//	wp_die( __('Sorry, can&#8217;t call files with their real path.' ));
+		// case 2 :
+		// wp_die( __('Sorry, can&#8217;t call files with their real path.' ));
 
 		case 3 :
 			wp_die( __( 'Sorry, that file cannot be edited.' ) );
@@ -191,145 +191,171 @@ function validate_file_to_edit( $file, $allowed_files = '' ) {
  * Handle PHP uploads in WordPress, sanitizing file names, checking extensions for mime type,
  * and moving the file to the appropriate directory within the uploads directory.
  *
- * @since 2.0.0
+ * @since 4.0.0
  *
- * @uses wp_handle_upload_error
- * @uses is_multisite
- * @uses wp_check_filetype_and_ext
- * @uses current_user_can
- * @uses wp_upload_dir
- * @uses wp_unique_filename
- * @uses delete_transient
- * @param array $file Reference to a single element of $_FILES. Call the function once for each uploaded file.
- * @param array $overrides Optional. An associative array of names=>values to override default variables with extract( $overrides, EXTR_OVERWRITE ).
- * @param string $time Optional. Time formatted in 'yyyy/mm'.
- * @return array On success, returns an associative array of file attributes. On failure, returns $overrides['upload_error_handler'](&$file, $message ) or array( 'error'=>$message ).
- */
-function wp_handle_upload( &$file, $overrides = false, $time = null ) {
+ * @see wp_handle_upload_error
+ *
+ * @param array  $file      Reference to a single element of $_FILES. Call the function once for
+ *                          each uploaded file.
+ * @param array  $overrides An associative array of names => values to override default variables.
+ * @param string $time      Time formatted in 'yyyy/mm'.
+ * @param string $action    Expected value for $_POST['action'].
+ * @return array On success, returns an associative array of file attributes. On failure, returns
+ *               $overrides['upload_error_handler'](&$file, $message ) or array( 'error'=>$message ).
+*/
+function _wp_handle_upload( &$file, $overrides, $time, $action ) {
 	// The default error handler.
 	if ( ! function_exists( 'wp_handle_upload_error' ) ) {
 		function wp_handle_upload_error( &$file, $message ) {
-			return array( 'error'=>$message );
+			return array( 'error' => $message );
 		}
 	}
 
 	/**
-	 * Filter data for the current file to upload.
+	 * The dynamic portion of the hook name, $action, refers to the post action.
 	 *
-	 * @since 2.9.0
+	 * @since 2.9.0 as 'wp_handle_upload_prefilter'
+	 * @since 4.0.0 Converted to a dynamic hook with $action
 	 *
 	 * @param array $file An array of data for a single file.
 	 */
-	$file = apply_filters( 'wp_handle_upload_prefilter', $file );
+	$file = apply_filters( "{$action}_prefilter", $file );
 
 	// You may define your own function and pass the name in $overrides['upload_error_handler']
 	$upload_error_handler = 'wp_handle_upload_error';
+	if ( isset( $overrides['upload_error_handler'] ) ) {
+		$upload_error_handler = $overrides['upload_error_handler'];
+	}
 
 	// You may have had one or more 'wp_handle_upload_prefilter' functions error out the file. Handle that gracefully.
-	if ( isset( $file['error'] ) && !is_numeric( $file['error'] ) && $file['error'] )
+	if ( isset( $file['error'] ) && ! is_numeric( $file['error'] ) && $file['error'] ) {
 		return $upload_error_handler( $file, $file['error'] );
+	}
+
+	// Install user overrides. Did we mention that this voids your warranty?
 
 	// You may define your own function and pass the name in $overrides['unique_filename_callback']
 	$unique_filename_callback = null;
+	if ( isset( $overrides['unique_filename_callback'] ) ) {
+		$unique_filename_callback = $overrides['unique_filename_callback'];
+	}
 
-	// $_POST['action'] must be set and its value must equal $overrides['action'] or this:
-	$action = 'wp_handle_upload';
-
-	// Courtesy of php.net, the strings that describe the error indicated in $_FILES[{form field}]['error'].
-	$upload_error_strings = array( false,
-		__( "The uploaded file exceeds the upload_max_filesize directive in php.ini." ),
-		__( "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form." ),
-		__( "The uploaded file was only partially uploaded." ),
-		__( "No file was uploaded." ),
-		'',
-		__( "Missing a temporary folder." ),
-		__( "Failed to write file to disk." ),
-		__( "File upload stopped by extension." ));
+	/*
+	 * This may not have orignially been intended to be overrideable,
+	 * but historically has been.
+	 */
+	if ( isset( $overrides['upload_error_strings'] ) ) {
+		$upload_error_strings = $overrides['upload_error_strings'];
+	} else {
+		// Courtesy of php.net, the strings that describe the error indicated in $_FILES[{form field}]['error'].
+		$upload_error_strings = array(
+			false,
+			__( 'The uploaded file exceeds the upload_max_filesize directive in php.ini.' ),
+			__( 'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form.' ),
+			__( 'The uploaded file was only partially uploaded.' ),
+			__( 'No file was uploaded.' ),
+			'',
+			__( 'Missing a temporary folder.' ),
+			__( 'Failed to write file to disk.' ),
+			__( 'File upload stopped by extension.' )
+		);
+	}
 
 	// All tests are on by default. Most can be turned off by $overrides[{test_name}] = false;
-	$test_form = true;
-	$test_size = true;
-	$test_upload = true;
+	$test_form = isset( $overrides['test_form'] ) ? $overrides['test_form'] : true;
+	$test_size = isset( $overrides['test_size'] ) ? $overrides['test_size'] : true;
 
-	// If you override this, you must provide $ext and $type!!!!
-	$test_type = true;
-	$mimes = false;
+	// If you override this, you must provide $ext and $type!!
+	$test_type = isset( $overrides['test_type'] ) ? $overrides['test_type'] : true;
+	$mimes = isset( $overrides['mimes'] ) ? $overrides['mimes'] : false;
 
-	// Install user overrides. Did we mention that this voids your warranty?
-	if ( is_array( $overrides ) )
-		extract( $overrides, EXTR_OVERWRITE );
+	$test_upload = isset( $overrides['test_upload'] ) ? $overrides['test_upload'] : true;
 
 	// A correct form post will pass this test.
-	if ( $test_form && (!isset( $_POST['action'] ) || ($_POST['action'] != $action ) ) )
-		return call_user_func($upload_error_handler, $file, __( 'Invalid form submission.' ));
-
+	if ( $test_form && ( ! isset( $_POST['action'] ) || ( $_POST['action'] != $action ) ) ) {
+		return call_user_func( $upload_error_handler, $file, __( 'Invalid form submission.' ) );
+	}
 	// A successful upload will pass this test. It makes no sense to override this one.
 	if ( isset( $file['error'] ) && $file['error'] > 0 ) {
 		return call_user_func( $upload_error_handler, $file, $upload_error_strings[ $file['error'] ] );
 	}
 
+	$test_file_size = 'wp_handle_upload' === $action ? $file['size'] : filesize( $file['tmp_name'] );
 	// A non-empty file will pass this test.
-	if ( $test_size && !($file['size'] > 0 ) ) {
-		if ( is_multisite() )
+	if ( $test_size && ! ( $test_file_size > 0 ) ) {
+		if ( is_multisite() ) {
 			$error_msg = __( 'File is empty. Please upload something more substantial.' );
-		else
+		} else {
 			$error_msg = __( 'File is empty. Please upload something more substantial. This error could also be caused by uploads being disabled in your php.ini or by post_max_size being defined as smaller than upload_max_filesize in php.ini.' );
-		return call_user_func($upload_error_handler, $file, $error_msg);
+		}
+		return call_user_func( $upload_error_handler, $file, $error_msg );
 	}
 
 	// A properly uploaded file will pass this test. There should be no reason to override this one.
-	if ( $test_upload && ! @ is_uploaded_file( $file['tmp_name'] ) )
-		return call_user_func($upload_error_handler, $file, __( 'Specified file failed upload test.' ));
+	$test_uploaded_file = 'wp_handle_upload' === $action ? @ is_uploaded_file( $file['tmp_name'] ) : @ is_file( $file['tmp_name'] );
+	if ( $test_upload && ! $test_uploaded_file ) {
+		return call_user_func( $upload_error_handler, $file, __( 'Specified file failed upload test.' ) );
+	}
 
 	// A correct MIME type will pass this test. Override $mimes or use the upload_mimes filter.
 	if ( $test_type ) {
 		$wp_filetype = wp_check_filetype_and_ext( $file['tmp_name'], $file['name'], $mimes );
-
-		extract( $wp_filetype );
+		$ext = empty( $wp_filetype['ext'] ) ? '' : $wp_filetype['ext'];
+		$type = empty( $wp_filetype['type'] ) ? '' : $wp_filetype['type'];
+		$proper_filename = empty( $wp_filetype['proper_filename'] ) ? '' : $wp_filetype['proper_filename'];
 
 		// Check to see if wp_check_filetype_and_ext() determined the filename was incorrect
-		if ( $proper_filename )
+		if ( $proper_filename ) {
 			$file['name'] = $proper_filename;
-
-		if ( ( !$type || !$ext ) && !current_user_can( 'unfiltered_upload' ) )
-			return call_user_func($upload_error_handler, $file, __( 'Sorry, this file type is not permitted for security reasons.' ));
-
-		if ( !$ext )
-			$ext = ltrim(strrchr($file['name'], '.'), '.');
-
-		if ( !$type )
+		}
+		if ( ( ! $type || !$ext ) && ! current_user_can( 'unfiltered_upload' ) ) {
+			return call_user_func( $upload_error_handler, $file, __( 'Sorry, this file type is not permitted for security reasons.' ) );
+		}
+		if ( ! $type ) {
 			$type = $file['type'];
+		}
 	} else {
 		$type = '';
 	}
 
-	// A writable uploads dir will pass this test. Again, there's no point overriding this one.
-	if ( ! ( ( $uploads = wp_upload_dir($time) ) && false === $uploads['error'] ) )
-		return call_user_func($upload_error_handler, $file, $uploads['error'] );
+	/*
+	 * A writable uploads dir will pass this test. Again, there's no point
+	 * overriding this one.
+	 */
+	if ( ! ( ( $uploads = wp_upload_dir( $time ) ) && false === $uploads['error'] ) ) {
+		return call_user_func( $upload_error_handler, $file, $uploads['error'] );
+	}
 
 	$filename = wp_unique_filename( $uploads['path'], $file['name'], $unique_filename_callback );
 
-	// Move the file to the uploads dir
+	// Move the file to the uploads dir.
 	$new_file = $uploads['path'] . "/$filename";
-	if ( false === @ move_uploaded_file( $file['tmp_name'], $new_file ) ) {
-		if ( 0 === strpos( $uploads['basedir'], ABSPATH ) )
-			$error_path = str_replace( ABSPATH, '', $uploads['basedir'] ) . $uploads['subdir'];
-		else
-			$error_path = basename( $uploads['basedir'] ) . $uploads['subdir'];
+	if ( 'wp_handle_upload' === $action ) {
+		$move_new_file = @ move_uploaded_file( $file['tmp_name'], $new_file );
+	} else {
+		$move_new_file = @ rename( $file['tmp_name'], $new_file );
+	}
 
+	if ( false === $move_new_file ) {
+		if ( 0 === strpos( $uploads['basedir'], ABSPATH ) ) {
+			$error_path = str_replace( ABSPATH, '', $uploads['basedir'] ) . $uploads['subdir'];
+		} else {
+			$error_path = basename( $uploads['basedir'] ) . $uploads['subdir'];
+		}
 		return $upload_error_handler( $file, sprintf( __('The uploaded file could not be moved to %s.' ), $error_path ) );
 	}
 
-	// Set correct file permissions
+	// Set correct file permissions.
 	$stat = stat( dirname( $new_file ));
 	$perms = $stat['mode'] & 0000666;
 	@ chmod( $new_file, $perms );
 
-	// Compute the URL
+	// Compute the URL.
 	$url = $uploads['url'] . "/$filename";
 
-	if ( is_multisite() )
+	if ( is_multisite() ) {
 		delete_transient( 'dirsize_cache' );
+	}
 
 	/**
 	 * Filter the data array for the uploaded file.
@@ -343,138 +369,69 @@ function wp_handle_upload( &$file, $overrides = false, $time = null ) {
 	 *     @type string $url  URL of the uploaded file.
 	 *     @type string $type File type.
 	 * }
-	 * @param string $context The type of upload action. Accepts 'upload' or 'sideload'.
+	 * @param string $context The type of upload action. Values include 'upload' or 'sideload'.
 	 */
-	return apply_filters( 'wp_handle_upload', array( 'file' => $new_file, 'url' => $url, 'type' => $type ), 'upload' );
+	return apply_filters( 'wp_handle_upload', array(
+		'file' => $new_file,
+		'url'  => $url,
+		'type' => $type
+	), 'wp_handle_sideload' === $action ? 'sideload' : 'upload' ); }
+
+/**
+ * Wrapper for _wp_handle_upload(), passes 'wp_handle_upload' action.
+ *
+ * @since 2.0.0
+ *
+ * @see _wp_handle_upload()
+ *
+ * @param array      $file      Reference to a single element of $_FILES. Call the function once for
+ *                              each uploaded file.
+ * @param array|bool $overrides Optional. An associative array of names=>values to override default
+ *                              variables. Default false.
+ * @param string     $time      Optional. Time formatted in 'yyyy/mm'. Default null.
+ * @return array On success, returns an associative array of file attributes. On failure, returns
+ *               $overrides['upload_error_handler'](&$file, $message ) or array( 'error'=>$message ).
+ */
+function wp_handle_upload( &$file, $overrides = false, $time = null ) {
+	/*
+	 *  $_POST['action'] must be set and its value must equal $overrides['action']
+	 *  or this:
+	 */
+	$action = 'wp_handle_upload';
+	if ( isset( $overrides['action'] ) ) {
+		$action = $overrides['action'];
+	}
+
+	return _wp_handle_upload( $file, $overrides, $time, $action );
 }
 
 /**
- * Handle sideloads, which is the process of retrieving a media item from another server instead of
- * a traditional media upload. This process involves sanitizing the filename, checking extensions
- * for mime type, and moving the file to the appropriate directory within the uploads directory.
+ * Wrapper for _wp_handle_upload(), passes 'wp_handle_sideload' action
  *
  * @since 2.6.0
  *
- * @uses wp_handle_upload_error
- * @uses wp_check_filetype_and_ext
- * @uses current_user_can
- * @uses wp_upload_dir
- * @uses wp_unique_filename
- * @param array $file an array similar to that of a PHP $_FILES POST array
- * @param array $overrides Optional. An associative array of names=>values to override default variables with extract( $overrides, EXTR_OVERWRITE ).
- * @param string $time Optional. Time formatted in 'yyyy/mm'.
- * @return array On success, returns an associative array of file attributes. On failure, returns $overrides['upload_error_handler'](&$file, $message ) or array( 'error'=>$message ).
+ * @see _wp_handle_upload()
+ *
+ * @param array      $file      An array similar to that of a PHP $_FILES POST array
+ * @param array|bool $overrides Optional. An associative array of names=>values to override default
+ *                              variables. Default false.
+ * @param string     $time      Optional. Time formatted in 'yyyy/mm'. Default null.
+ * @return array On success, returns an associative array of file attributes. On failure, returns
+ *               $overrides['upload_error_handler'](&$file, $message ) or array( 'error'=>$message ).
  */
 function wp_handle_sideload( &$file, $overrides = false, $time = null ) {
-	// The default error handler.
-	if (! function_exists( 'wp_handle_upload_error' ) ) {
-		function wp_handle_upload_error( &$file, $message ) {
-			return array( 'error'=>$message );
-		}
-	}
-
-	// You may define your own function and pass the name in $overrides['upload_error_handler']
-	$upload_error_handler = 'wp_handle_upload_error';
-
-	// You may define your own function and pass the name in $overrides['unique_filename_callback']
-	$unique_filename_callback = null;
-
-	// $_POST['action'] must be set and its value must equal $overrides['action'] or this:
+	/*
+	 *  $_POST['action'] must be set and its value must equal $overrides['action']
+	 *  or this:
+	 */
 	$action = 'wp_handle_sideload';
-
-	// Courtesy of php.net, the strings that describe the error indicated in $_FILES[{form field}]['error'].
-	$upload_error_strings = array( false,
-		__( "The uploaded file exceeds the <code>upload_max_filesize</code> directive in <code>php.ini</code>." ),
-		__( "The uploaded file exceeds the <em>MAX_FILE_SIZE</em> directive that was specified in the HTML form." ),
-		__( "The uploaded file was only partially uploaded." ),
-		__( "No file was uploaded." ),
-		'',
-		__( "Missing a temporary folder." ),
-		__( "Failed to write file to disk." ),
-		__( "File upload stopped by extension." ));
-
-	// All tests are on by default. Most can be turned off by $overrides[{test_name}] = false;
-	$test_form = true;
-	$test_size = true;
-
-	// If you override this, you must provide $ext and $type!!!!
-	$test_type = true;
-	$mimes = false;
-
-	// Install user overrides. Did we mention that this voids your warranty?
-	if ( is_array( $overrides ) )
-		extract( $overrides, EXTR_OVERWRITE );
-
-	// A correct form post will pass this test.
-	if ( $test_form && (!isset( $_POST['action'] ) || ($_POST['action'] != $action ) ) )
-		return $upload_error_handler( $file, __( 'Invalid form submission.' ));
-
-	// A successful upload will pass this test. It makes no sense to override this one.
-	if ( ! empty( $file['error'] ) )
-		return $upload_error_handler( $file, $upload_error_strings[$file['error']] );
-
-	// A non-empty file will pass this test.
-	if ( $test_size && !(filesize($file['tmp_name']) > 0 ) )
-		return $upload_error_handler( $file, __( 'File is empty. Please upload something more substantial. This error could also be caused by uploads being disabled in your php.ini.' ));
-
-	// A properly uploaded file will pass this test. There should be no reason to override this one.
-	if (! @ is_file( $file['tmp_name'] ) )
-		return $upload_error_handler( $file, __( 'Specified file does not exist.' ));
-
-	// A correct MIME type will pass this test. Override $mimes or use the upload_mimes filter.
-	if ( $test_type ) {
-		$wp_filetype = wp_check_filetype_and_ext( $file['tmp_name'], $file['name'], $mimes );
-
-		extract( $wp_filetype );
-
-		// Check to see if wp_check_filetype_and_ext() determined the filename was incorrect
-		if ( $proper_filename )
-			$file['name'] = $proper_filename;
-
-		if ( ( !$type || !$ext ) && !current_user_can( 'unfiltered_upload' ) )
-			return $upload_error_handler( $file, __( 'Sorry, this file type is not permitted for security reasons.' ));
-
-		if ( !$ext )
-			$ext = ltrim(strrchr($file['name'], '.'), '.');
-
-		if ( !$type )
-			$type = $file['type'];
+	if ( isset( $overrides['action'] ) ) {
+		$action = $overrides['action'];
 	}
-
-	// A writable uploads dir will pass this test. Again, there's no point overriding this one.
-	if ( ! ( ( $uploads = wp_upload_dir( $time ) ) && false === $uploads['error'] ) )
-		return $upload_error_handler( $file, $uploads['error'] );
-
-	$filename = wp_unique_filename( $uploads['path'], $file['name'], $unique_filename_callback );
-
-	// Strip the query strings.
-	$filename = str_replace('?','-', $filename);
-	$filename = str_replace('&','-', $filename);
-
-	// Move the file to the uploads dir
-	$new_file = $uploads['path'] . "/$filename";
-	if ( false === @ rename( $file['tmp_name'], $new_file ) ) {
-		if ( 0 === strpos( $uploads['basedir'], ABSPATH ) )
-			$error_path = str_replace( ABSPATH, '', $uploads['basedir'] ) . $uploads['subdir'];
-		else
-			$error_path = basename( $uploads['basedir'] ) . $uploads['subdir'];
-		return $upload_error_handler( $file, sprintf( __('The uploaded file could not be moved to %s.' ), $error_path ) );
-	}
-
-	// Set correct file permissions
-	$stat = stat( dirname( $new_file ));
-	$perms = $stat['mode'] & 0000666;
-	@ chmod( $new_file, $perms );
-
-	// Compute the URL
-	$url = $uploads['url'] . "/$filename";
-
-	/** This filter is documented in wp-admin/includes/file.php */
-	$return = apply_filters( 'wp_handle_upload', array( 'file' => $new_file, 'url' => $url, 'type' => $type ), 'sideload' );
-
-	return $return;
+	return _wp_handle_upload( $file, $overrides, $time, $action );
 }
 
+
 /**
  * Downloads a url to a local temporary file using the WordPress HTTP Class.
  * Please note, That the calling function must unlink() the file.
@@ -1041,7 +998,7 @@ function request_filesystem_credentials($form_post, $type = '', $error = false,
 	$credentials['public_key'] = defined('FTP_PUBKEY') ? FTP_PUBKEY : (!empty($submitted_form['public_key']) ? $submitted_form['public_key'] : '');
 	$credentials['private_key'] = defined('FTP_PRIKEY') ? FTP_PRIKEY : (!empty($submitted_form['private_key']) ? $submitted_form['private_key'] : '');
 
-	//sanitize the hostname, Some people might pass in odd-data:
+	// Sanitize the hostname, Some people might pass in odd-data:
 	$credentials['hostname'] = preg_replace('|\w+://|', '', $credentials['hostname']); //Strip any schemes off
 
 	if ( strpos($credentials['hostname'], ':') ) {
@@ -1074,12 +1031,13 @@ function request_filesystem_credentials($form_post, $type = '', $error = false,
 		update_option('ftp_credentials', $stored_credentials);
 		return $credentials;
 	}
-	$hostname = '';
-	$username = '';
-	$password = '';
-	$connection_type = '';
-	if ( !empty($credentials) )
-		extract($credentials, EXTR_OVERWRITE);
+	$hostname = isset( $credentials['hostname'] ) ? $credentials['hostname'] : '';
+	$username = isset( $credentials['username'] ) ? $credentials['username'] : '';
+	$public_key = isset( $credentials['public_key'] ) ? $credentials['public_key'] : '';
+	$private_key = isset( $credentials['private_key'] ) ? $credentials['private_key'] : '';
+	$port = isset( $credentials['port'] ) ? $credentials['port'] : '';
+	$connection_type = isset( $credentials['connection_type'] ) ? $credentials['connection_type'] : '';
+
 	if ( $error ) {
 		$error_string = __('<strong>ERROR:</strong> There was an error connecting to the server, Please verify the settings are correct.');
 		if ( is_wp_error($error) )
@@ -1169,7 +1127,8 @@ jQuery(function($){
 <label for="public_key"><?php _e('Public Key:') ?></label ><br />
 <label for="private_key"><?php _e('Private Key:') ?></label>
 </div></th>
-<td><br /><input name="public_key" type="text" id="public_key" value="<?php echo esc_attr($public_key) ?>"<?php disabled( defined('FTP_PUBKEY') ); ?> size="40" /><br /><input name="private_key" type="text" id="private_key" value="<?php echo esc_attr($private_key) ?>"<?php disabled( defined('FTP_PRIKEY') ); ?> size="40" />
+<td><br /><input name="public_key" type="text" id="public_key" value="<?php echo esc_attr($public_key) ?>"<?php disabled( defined('FTP_PUBKEY') ); ?> size="40" />
+	<br /><input name="private_key" type="text" id="private_key" value="<?php echo esc_attr($private_key) ?>"<?php disabled( defined('FTP_PRIKEY') ); ?> size="40" />
 <div><?php _e('Enter the location on the server where the keys are located. If a passphrase is needed, enter that in the password field above.') ?></div></td>
 </tr>
 <?php endif; ?>
